import glm

# ciselne oznacenie smeru posunu Avatara
goForward: int = 0
goBack: int = 1
goUp: int = 2
goDown: int = 3
goRight: int = 4
goLeft: int = 5


class Avatar:
    def __init__(self, *args, **kwargs):
        self.pos = glm.vec3(0.0, 0.0, 0.0)      # default hodnota
        self.scr_ratio: float = 800.0 / 600.0   # default hodnota
        self.Yaw: float = -90.0                 # default hodnota
        self.Pitch: float = 0.0                 # default hodnota
        self.FOV: float = 60.0                  # startovacia hodnota
        self.Front = glm.vec3(0.0, 0.0, -1.0)   # startovacia hodnota
        self.Right = glm.vec3()                 # startovacia hodnota
        self.Up = glm.vec3()                    # startovacia hodnota
        self.WorldUp = glm.vec3(0.0, 1.0, 0.0)  # const
        self.speed: float = 10.0                # const
        self.viewnear: float = 0.1              # const
        self.viewfar: float = 1000.0            # const

        if len(args) == 6 and len(kwargs) == 0:
            posX, posY, posZ, scr_ratio, yaw, pitch = args
            self.pos = glm.vec3(posX, posY, posZ)
            self.scr_ratio = scr_ratio
            self.Yaw = yaw
            self.Pitch = pitch
        elif len(args) + len(kwargs) <= 4:
            keyword_arguments = ("position", "scr_ratio", "yaw", "pitch")
            keyword_arguments_defaults = {"position": glm.vec3(), "scr_ratio": self.scr_ratio,
                                          "yaw": self.Yaw, "pitch": self.Pitch}
            for i in range(len(args)):
                kwa = keyword_arguments[i]
                value = args[i]
                kwargs[kwa] = value
            keyword_arguments_defaults.update(kwargs)
            self.pos = keyword_arguments_defaults["position"]
            self.scr_ratio = keyword_arguments_defaults["scr_ratio"]
            self.Yaw = keyword_arguments_defaults["yaw"]
            self.Pitch = keyword_arguments_defaults["pitch"]
        else:
            raise TypeError("Nespravny pocet argumentov pri vytvoreni instancie triedy Avatar()!")

        # vypocet vektorov Front, Right a Up
        self.update()

    # vypocet vektorov Front, Right a Up -----------------------------------------
    def update(self) -> None:
        front = glm.vec3()
        front.x = glm.cos(glm.radians(self.Yaw)) * glm.cos(glm.radians(self.Pitch))
        front.y = glm.sin(glm.radians(self.Pitch))
        front.z = glm.sin(glm.radians(self.Yaw)) * glm.cos(glm.radians(self.Pitch))
        self.Front = glm.normalize(front)
        self.Right = glm.normalize(glm.cross(self.Front, self.WorldUp))
        self.Up = glm.normalize(glm.cross(self.Right, self.Front))

    # projekcna matica -----------------------------------------------------------
    def get_proj_mat(self) -> glm.mat4:
        return glm.perspective(glm.radians(self.FOV), self.scr_ratio, self.viewnear, self.viewfar)

    # modelview matica -----------------------------------------------------------
    def get_mv_mat(self) -> glm.mat4:
        return glm.lookAt(self.pos, self.pos + self.Front, self.Up)

    # nastavenie polohy Avatara --------------------------------------------------
    def setpos(self, mDir: int, dTime: float) -> None:
        if mDir == goForward:
            self.pos += self.Front * self.speed * dTime
        elif mDir == goBack:
            self.pos -= self.Front * self.speed * dTime
        elif mDir == goLeft:
            self.pos -= self.Right * self.speed * dTime
        elif mDir == goRight:
            self.pos += self.Right * self.speed * dTime

    # nastavenie natocenia Avatara -----------------------------------------------
    def setrot(self, xoffset: float, yoffset: float) -> None:
        self.Yaw += 0.1 * xoffset
        self.Pitch += 0.1 * yoffset
        if self.Pitch > 89.0:
            self.Pitch = 89.0
        elif self.Pitch < -89.0:
            self.Pitch = -89.0
        # vypocet vektorov Front, Right a Up
        self.update()

    # nastavenie FOV Avatara -----------------------------------------------------
    def setFOV(self, yoffset: float) -> None:
        self.FOV -= 2.0 * yoffset
        if self.FOV > 60.0:
            self.FOV = 60.0
        elif self.FOV < 1.0:
            self.FOV = 1.0
