import glm

from OpenGL.GL import *


def check_Errors(shaderid: int, strname: str) -> None:
    if strname != "program":
        status = glGetShaderiv(shaderid, GL_COMPILE_STATUS)
        if not status:
            infoLog = glGetShaderInfoLog(shaderid)
            print("Chyba kompilacie shadera: " + strname + "\n" + infoLog.decode() + "\n")
    else:
        status = glGetProgramiv(shaderid, GL_LINK_STATUS)
        if not status:
            infoLog = glGetProgramInfoLog(shaderid)
            print("Chyba linkovania programu: " + strname + "\n" + infoLog.decode() + "\n")


class Shader:
    def __init__(self, vspath: str, fspath: str):
        try:
            vsFile = open(vspath)  # kod vertex shadera
            vsCode = vsFile.read()
            vsFile.close()
            fsFile = open(fspath)  # kod fragment shadera
            fsCode = fsFile.read()
            fsFile.close()

            # vytvorenie shader objektu (kontajnera)
            vertex = glCreateShader(GL_VERTEX_SHADER)
            glShaderSource(vertex, vsCode)
            glCompileShader(vertex)  # kompilacia VS
            check_Errors(vertex, "vertex shader")
            # vytvorenie shader objektu (kontajnera)
            fragment = glCreateShader(GL_FRAGMENT_SHADER)
            glShaderSource(fragment, fsCode)
            glCompileShader(fragment)  # kompilacia FS
            check_Errors(fragment, "fragment shader")

            # vytvorenie program objektu (kontajnera)
            self.program = glCreateProgram()
            # pripojenie shaderov
            glAttachShader(self.program, vertex)
            glAttachShader(self.program, fragment)
            glLinkProgram(self.program)  # linkovanie PROGRAMU
            check_Errors(self.program, "program")

            # odpojenie shaderov
            glDetachShader(self.program, vertex)
            glDetachShader(self.program, fragment)
            # zrusenie shader objektov
            glDeleteShader(vertex)
            glDeleteShader(fragment)
        except IOError:
            print("Nemozno nacitat kod GLSL shaderov!")

    def use(self) -> None:
        glUseProgram(self.program)

    def set_U1i(self, strname: str, i: int) -> None:
        # umiestnenie, hodnota
        glUniform1i(glGetUniformLocation(self.program, strname), i)

    def set_UMatrix4fv(self, strname: str, m: glm.mat4) -> None:
        # umiestnenie, 1 matica, transponovat, hodnota
        glUniformMatrix4fv(glGetUniformLocation(self.program, strname), 1, GL_FALSE, glm.value_ptr(m))
